/* Copyright 1996-1999, Jan D. Wolter and Steven R. Weiss, All Rights Reserved.
 */

#ifndef _BT_COMMON_H
#define _BT_COMMON_H

#include <stdio.h>
#include "config.h"
#include "version.h"
#include <sys/types.h>
#include <utmp.h>

#ifdef STDC_HEADERS
#include <string.h>
#include <stdlib.h>
#else
#ifndef HAVE_STRCHR
#define strchr index
#define strrchr rindex
#endif
char *strchr(), *strdup(), *strrchr(), *strstr(), *getenv();
void *malloc(), *realloc();
#endif

#define DFLT_CFLIST BBS_DIR"dflt.cflist"
#define CONFLIST BBS_DIR"conflist"
#define CONFINDEX BBS_DIR"confindex"
#define CENSOR_LOG BBS_DIR"censored"
#define IDENT_FILE "id"

#define INDEX_SUBDIR "indexdir"
#define INDEX_PREFIX "@"

#define MAXSYMLEN  32	/* max length of symbol names in internal dictionary */
#define BFSZ 1024	/* standard buffer size for various uses */

/* If using real unix logins, use real login name length */
#if defined(UD_PASSWD) && !defined(PASSWD_FILE)
#define AUTH_UNIX
#define ALLOW_CFDIR
#ifdef UT_NAMELEN
#ifdef MAX_LOGIN_LEN
#undef MAX_LOGIN_LEN
#endif
#define MAX_LOGIN_LEN UT_NAMELEN
#endif
#endif

#ifdef PICOSPAN_COMPAT
#define HIDE_HTML
#else
#define PARENT_POINTERS
#endif

#ifdef YAPP_COMPAT
#define HIDE_HTML
#define KEEP_ULIST
#define CONF_LOG
#endif

#if defined(UD_PASSWD) && !defined(PASSWD_FILE)
# define NOPWEDIT 1      /* Cannot edit password files on real unix accouts */
#else
# define NOPWEDIT 0
#endif

#define DFLT_FW "cfadm"

#define TRUE    1
#define FALSE   0

/* Standardize some defines that may be missing on systems */

#ifdef _PATH_LASTLOG
#define LASTLOG _PATH_LASTLOG
#else
#define LASTLOG "/usr/adm/lastlog"
#endif

#ifndef UT_NAMESIZE
#define UT_NAMESIZE 8
#endif


#ifdef DEBUG_MEMORY
#define malloc(s) xmalloc(s)
#define realloc(p,s) xrealloc(p,s)
#define free(p) xfree(p)
#define MYSTRDUP
void *xmalloc(size_t);
void *xrealloc(void *,size_t);
void *xfree(void *);
#endif

/* Variadic functions.  We borrow code from Gnu Make to go through contortions
 * to allow proper function prototypes for both ANSI and pre-ANSI C compilers,
 * and also for those which support stdarg.h vs. varargs.h, and finally for
 * those which have vfprintf(), etc. and those who have _doprnt().. or nothing.
 * The defines for the case where you don't have stdarg.h or varargs.h are
 * borderline flakey.
 */

#if defined(HAVE_VPRINTF) || defined(HAVE_DOPRNT)
#define HAVE_STDVARARGS 1
#if __STDC__
#include <stdarg.h>
#define VA_START(args, lastarg) va_start(args, lastarg)
#else
#include <varargs.h>
#define VA_START(args, lastarg) va_start(args)
#endif /* __STDC__ */
#ifdef HAVE_VPRINTF
#define VA_PRINTF(fp, fmt, args) vfprintf((fp), (fmt), (args))
#else
#define VA_PRINTF(fp, fmt, args) _doprnt((fmt), (args), (fp))
#endif /* HAVE_VPRINTF */
#define VA_ARG(args, type) va_arg((args), type)
#define VA_ARG1(args, type) va_arg((args), type) 
#define VA_ARG2(args, type) va_arg((args), type)
#define VA_ARG3(args, type) va_arg((args), type)
#define VA_ARG4(args, type) va_arg((args), type)
#define VA_END(args) va_end(args)
#else
#define HAVE_STDVARARGS 0
#define va_alist a1, a2, a3, a4, a5, a6, a7, a8
#define va_dcl char *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8;
#define VA_START(args, lastarg)
#define VA_PRINTF(fp, fmt, args) fprintf((fp), (fmt), va_alist)
/* I haven't figured out a definition for VA_ARG in this case.  The ones
 * below only work if all the arguments are the same sizes as pointers. */
#define VA_ARG1(args,type) ((type)a1)
#define VA_ARG2(args,type) ((type)a2)
#define VA_ARG3(args,type) ((type)a3)
#define VA_ARG4(args,type) ((type)a4)
#define VA_END(args)
#endif /* HAVE_VPRINTF || HAVE_DOPRNT */


#endif /* _BT_COMMON_H */
